library enx_uikit_flutter;

import 'dart:io';

import 'package:animated_widgets/widgets/translation_animated.dart';
import 'package:draggable_widget/draggable_widget.dart';
import 'package:draggable_widget/model/anchor_docker.dart';
import 'package:enx_flutter_plugin/enx_player_widget.dart';
import 'package:enx_uikit_flutter/screens/app_bar.dart';
import 'package:enx_uikit_flutter/screens/floating_layout.dart';
import 'package:enx_uikit_flutter/screens/grid_layout.dart';
import 'package:enx_uikit_flutter/screens/participants_list.dart';
import 'package:enx_uikit_flutter/utilities/color.dart';
import 'package:enx_uikit_flutter/utilities/custom_dialog.dart';
import 'package:enx_uikit_flutter/widgets/blink_icon.dart';
import 'package:enx_uikit_flutter/widgets/enx_loader.dart';
import 'package:enx_uikit_flutter/widgets/more_widget.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:get/get.dart';

import 'buttons/enx_button.dart';
import 'controller/enx_controller.dart';

//EnxVideo View .
class EnxVideoView extends StatelessWidget {
  final String token;
  final void Function(Map<dynamic, dynamic> map) disconnect;
  final void Function(Map<dynamic, dynamic> map) connectError;
  EnxVideoView({
    Key? key,
    required this.token,
    required this.disconnect,
    required this.connectError,
  }) : super(key: key);

  Widget localView() {
    return SizedBox(
      height: 150.h,
      width: 100.w,
      child: EnxPlayerWidget(
        0,
        local: true,
        width: 100.w.toInt(),
        height: 150.h.toInt(),
        zMediaOverlay: true,
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final obj =
        Get.put(EnxController(token, context, disconnect, connectError));
    ScreenUtil.init(
      BoxConstraints(
          maxWidth: MediaQuery.of(context).size.width,
          maxHeight: MediaQuery.of(context).size.height),
      designSize: Size(MediaQuery.of(context).size.width,
          MediaQuery.of(context).size.height),
      context: context,
    );

    return InkWell(
      onDoubleTap: () {
        print("ontap");
        if (obj.isShowHide) {
          obj.isShow.value = false;
        } else {
          obj.isShow.value = true;
        }
      },
      child: Obx(() => Scaffold(
            backgroundColor: Colors.black,
            extendBodyBehindAppBar: false,
            body: SafeArea(
              child: Platform.isIOS
                  ? prepareView(obj, context)
                  : obj.dataAvailable
                      ? prepareView(obj, context)
                      : Center(
                          child: Container(
                            padding: EdgeInsets.all(16.w),
                            child: EnxLoader.getHeading(obj.loading),
                          ),
                        ),
            ),
          )),
    );
  }

  Widget prepareView(EnxController obj, BuildContext context) {
    return Stack(
      alignment: Alignment.center,
      children: [
        Center(
          child: obj.activeTalkerList.isEmpty
              ? Text(
                  'Wait for others to join',
                  style: TextStyle(color: Colors.white, fontSize: 16.sp),
                )
              : Container(
                  child: obj.isGridView
                      ? MediaQuery.of(context).orientation ==
                              Orientation.portrait
                          ? Column(
                              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                              children: renderStreamsGrid(
                                  obj.activeTalkerList, MediaQuery.of(context)))
                          : Row(
                              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                              children: renderStreamsGrid(
                                  obj.activeTalkerList, MediaQuery.of(context)))
                      : viewFloat(context, obj),
                ),
        ),
        TranslationAnimatedWidget(
          enabled: obj.isShowHide, //will forward/reverse the animation
          curve: Curves.easeIn,
          duration: const Duration(seconds: 1),

          child: EnxVideoButtons(
            enxController: obj,
          ),
        ),
        Column(
          children: [
            TranslationAnimatedWidget(
              enabled: obj.isShowHide, //will forward/reverse the animation
              curve: Curves.easeIn,
              duration: const Duration(seconds: 1),
              values: [Offset(0, 0), Offset(0, -150)],
              child: Align(
                  alignment: Alignment.topCenter,
                  child: Padding(
                    padding: const EdgeInsets.only(top: 20.0),
                    child: CustomAppBar(
                      appMode: obj.mode,
                      role: obj.isRole,
                      floorAction: obj.floorAction,
                      moreCallbacks: (String map) {
                        showMoreDialog(context, obj);
                      },
                      participantsCallbacks: () {
                        Get.to(() => ParticipantScreen(obj),
                            transition: Transition.leftToRight,
                            duration: const Duration(seconds: 1));
                      },
                      raiseHandCallbacks: () {
                        switch (obj.floorAction) {
                          case 0:
                            if (obj.isRole) {
                              setupAlertDialogContainer(context, obj);
                            } else {
                              obj.requestFloors();
                            }
                            break;
                          case 1:
                            var baseDialog = BaseAlertDialog(
                                title: "Cancel Floor",
                                content: "You want to cancel floor request",
                                yesOnPressed: () {
                                  obj.cancelFloors();
                                  obj.participantFloorAction.value = 0;
                                  Get.back();
                                },
                                noOnPressed: () {
                                  Get.back();
                                },
                                yes: "Yes",
                                no: "Cancel");
                            showDialog(
                                context: context,
                                builder: (BuildContext context) => baseDialog);

                            break;
                          case 2:
                            var baseDialog = BaseAlertDialog(
                                title: "Finish Floor",
                                content: "You want to finish floor request",
                                yesOnPressed: () {
                                  obj.finishFloors();
                                  obj.participantFloorAction.value = 0;
                                  Get.back();
                                },
                                noOnPressed: () {
                                  Get.back();
                                },
                                yes: "Finish",
                                no: "Cancel");
                            showDialog(
                                context: context,
                                builder: (BuildContext context) => baseDialog);
                            break;
                        }
                      },
                    ),
                  )),
            ),
            Visibility(
              visible: obj.isRecord ? true : false,
              child: Align(
                alignment: Alignment.topCenter,
                child: Padding(
                  padding: EdgeInsets.only(top: 10.h, right: 20.h),
                  child: BlinkingPoint(
                    xCoor: 20.0, // The x coordinate of the point
                    yCoor: 20.0, // The y coordinate of the point
                    pointColor:
                        CustomColors.themeColor, // The color of the point
                    pointSize: 8.0.w, // The size of the point)
                  ),
                ),
              ),
            ),
          ],
        ),
        DraggableWidget(
          bottomMargin: 50.w,
          topMargin: 80.w,
          intialVisibility: true,
          horizontalSpace: 20.w,
          shadowBorderRadius: 10.w,
          child: localView(),
          initialPosition: AnchoringPosition.topRight,
        ),
      ],
    );
  }

  void setupAlertDialogContainer(context, EnxController obj) {
    showDialog(
        context: context,
        builder: (BuildContext context) {
          return obj.floorRequestList.isEmpty
              ? SimpleDialog(children: [
                  Center(
                      child: Text(
                    'No Request found',
                    style: TextStyle(color: Colors.black, fontSize: 17.sp),
                  ))
                ])
              : Dialog(
                  insetPadding: const EdgeInsets.all(10),
                  child: ListView(shrinkWrap: true, children: [
                    for (var item in obj.floorRequestList)
                      ListTile(
                        leading: const Icon(Icons.person_outlined),
                        title: Text(
                          item.name,
                        ),
                        trailing: Row(
                          mainAxisSize: MainAxisSize.min,
                          children: [
                            Visibility(
                              visible: item.isRequestAccepted,
                              child: ElevatedButton(
                                onPressed: () {
                                  obj.acceptFloors(item.clientId);
                                  Get.back();
                                },
                                child: const Text('Accept'),
                                style: ButtonStyle(
                                    backgroundColor:
                                        MaterialStateProperty.all<Color>(
                                            Colors.red),
                                    shape: MaterialStateProperty.all<
                                            RoundedRectangleBorder>(
                                        RoundedRectangleBorder(
                                            borderRadius:
                                                BorderRadius.circular(12.0),
                                            side: const BorderSide(
                                                color: Colors.red)))),
                              ),
                            ),
                            Visibility(
                              visible: item.isRequestRejected,
                              child: Padding(
                                padding: const EdgeInsets.only(left: 2.0),
                                child: ElevatedButton(
                                  onPressed: () {
                                    obj.rejectFloors(item.clientId);
                                    Get.back();
                                  },
                                  child: const Text('Reject'),
                                  style: ButtonStyle(
                                      backgroundColor:
                                          MaterialStateProperty.all<Color>(
                                              Colors.red),
                                      shape: MaterialStateProperty.all<
                                              RoundedRectangleBorder>(
                                          RoundedRectangleBorder(
                                              borderRadius:
                                                  BorderRadius.circular(12.0),
                                              side: const BorderSide(
                                                  color: Colors.red)))),
                                ),
                              ),
                            ),
                            Visibility(
                                visible: item.isRequestReleased,
                                child: Padding(
                                  padding: const EdgeInsets.only(left: 2.0),
                                  child: ElevatedButton(
                                    onPressed: () {
                                      obj.releaseFloors(item.clientId);
                                      Get.back();
                                    },
                                    child: const Text('Revoke'),
                                    style: ButtonStyle(
                                        backgroundColor:
                                            MaterialStateProperty.all<Color>(
                                                Colors.red),
                                        shape: MaterialStateProperty.all<
                                                RoundedRectangleBorder>(
                                            RoundedRectangleBorder(
                                                borderRadius:
                                                    BorderRadius.circular(12.0),
                                                side: const BorderSide(
                                                    color: Colors.red)))),
                                  ),
                                )),
                          ],
                        ),
                      ),
                  ]));
        });
  }
}
