import 'package:enx_uikit_flutter/controller/enx_controller.dart';
import 'package:enx_uikit_flutter/models/ChatModel.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:get/get_core/src/get_main.dart';
import 'package:get/get_navigation/src/extension_navigation.dart';

class ChatPage extends StatelessWidget {
  EnxController enxController;
  String type;


  String? clientId;

  ChatPage(this.enxController, this.type, {Key? key, this.clientId})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    print("12111${type}");
    return Scaffold(
      backgroundColor: Colors.white,
      body: SafeArea(
        child: Obx(
          () => Stack(
            children: [
              Column(
                children: [
                  _topChat(),
                  _bodyChat(),
                  const SizedBox(
                    height: 120,
                  )
                ],
              ),
              _formChat(),
            ],
          ),
        ),
      ),
    );
  }

  _topChat() {
    return Container(
      padding: EdgeInsets.symmetric(horizontal: 30, vertical: 15),
      child: Row(
        children: [
          GestureDetector(
            onTap: () => Get.back(),
            child: Icon(
              Icons.arrow_back_ios,
              size: 25,
              color: Colors.black87,
            ),
          ),
          Text(
            type == 'group' ? 'Group Chat' : type,
            style: TextStyle(
                fontSize: 20,
                fontWeight: FontWeight.bold,
                color: Colors.black87),
          ),
        ],
      ),
    );
  }

  List<ChatModel> getPrivateChat() {
    for (var chatData in enxController.privateChatListData) {
      if (chatData.clientId == clientId) {
        print("er12${chatData.chatList.length}");
        return chatData.chatList;
      }
    }
    return enxController.privateChatList;
  }

  Widget _bodyChat() {
    return Expanded(
      child: Container(
        padding: const EdgeInsets.only(left: 25, right: 25, top: 25),
        width: double.infinity,
        decoration: const BoxDecoration(
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(20), topRight: Radius.circular(20)),
          color: Colors.white,
        ),
        child: ListView(
          controller:enxController.scrollController ,
          shrinkWrap: true,
          physics: ClampingScrollPhysics(),
          children: [
            for (var item
                in type == 'group' ? enxController.chatList : getPrivateChat())
              Column(
                crossAxisAlignment: CrossAxisAlignment.end,
                mainAxisAlignment: MainAxisAlignment.start,
                children: [
                  Row(
                    mainAxisAlignment: item.recieved!
                        ? MainAxisAlignment.end
                        : MainAxisAlignment.start,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(
                        '${item.recieved! ? item.sender : 'Me'}',
                        style: TextStyle(color: Colors.grey.shade400),
                      ),
                      Flexible(
                        child: Container(
                          margin: const EdgeInsets.only(
                              left: 2, right: 10, top: 10),
                          padding: const EdgeInsets.all(20),
                          decoration: BoxDecoration(
                            color: item.recieved!
                                ? Colors.indigo.shade100
                                : Colors.indigo.shade50,
                            borderRadius: item.broadcast
                                ? const BorderRadius.only(
                                    topLeft: Radius.circular(10),
                                    topRight: Radius.circular(10),
                                    bottomLeft: Radius.circular(10),
                                  )
                                : const BorderRadius.only(
                                    topLeft: Radius.circular(10),
                                    topRight: Radius.circular(10),
                                    bottomRight: Radius.circular(10),
                                  ),
                          ),
                          child: Text('${item.message}'),
                        ),
                      ),
                    ],
                  ),
                  Align(
                    alignment: item.recieved!
                        ? Alignment.bottomRight
                        : Alignment.bottomLeft,
                    child: Text(
                      '${item.currentTime}',
                      style: TextStyle(color: Colors.grey.shade400),
                      textAlign: TextAlign.justify,
                    ),
                  )
                ],
              )
          ],
        ),
      ),
    );
  }

  TextEditingController msgCtr = TextEditingController();

  Widget _formChat() {
    return Positioned(
      child: Align(
        alignment: Alignment.bottomCenter,
        child: Container(
          height: 120,
          padding: EdgeInsets.symmetric(vertical: 30, horizontal: 20),
          color: Colors.white,
          child: TextField(
            controller: msgCtr,
            decoration: InputDecoration(
              hintText: 'Type your message...',
              suffixIcon: Container(
                decoration: BoxDecoration(
                    borderRadius: BorderRadius.circular(40), color: Colors.red),
                padding: const EdgeInsets.all(5),
                child: IconButton(
                  icon: Icon(
                    Icons.send_rounded,
                    color: Colors.white,
                    size: 25,
                  ),
                  onPressed: () {
                    // msgCtr.text = "";
                    if (msgCtr.text.toString().isNotEmpty) {
                      if (type == 'group') {
                        enxController.sendMessage(
                            msgCtr.text.toString(), true, [], type);
                      } else {
                        enxController.sendMessage(
                            msgCtr.text.toString(), false, [clientId!], type);
                      }


                      print('ontape');
                      msgCtr.text = "";
                    } else {}
                  },
                ),
              ),
              filled: true,
              fillColor: Colors.blueGrey[50],
              labelStyle: TextStyle(fontSize: 12),
              contentPadding: EdgeInsets.all(20),
              enabledBorder: OutlineInputBorder(
                borderSide: BorderSide(color: Colors.blueGrey),
                borderRadius: BorderRadius.circular(25),
              ),
              focusedBorder: OutlineInputBorder(
                borderSide: BorderSide(color: Colors.blueGrey),
                borderRadius: BorderRadius.circular(25),
              ),
            ),
          ),
        ),
      ),
    );
  }
}

class Avatar extends StatelessWidget {
  final double size;
  final image;
  final EdgeInsets margin;

  Avatar({this.image, this.size = 50, this.margin = const EdgeInsets.all(0)});

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: margin,
      child: Container(
        width: size,
        height: size,
        decoration: new BoxDecoration(
          shape: BoxShape.circle,
          image: new DecorationImage(
            image: AssetImage(
              'assets/image/chat_icon_color.png',
              package: 'enx_uikit_flutter',
            ),
          ),
        ),
      ),
    );
  }
}
