import 'package:enx_uikit_flutter/buttons/custom_button.dart';
import 'package:enx_uikit_flutter/controller/enx_controller.dart';
import 'package:enx_uikit_flutter/utilities/color.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:get/get_core/src/get_main.dart';
import 'package:get/get_instance/src/extension_instance.dart';
import 'package:get/get_navigation/src/extension_navigation.dart';
import 'package:get/get_navigation/src/routes/transitions_type.dart';
import 'package:get/get_state_manager/src/rx_flutter/rx_obx_widget.dart';

import 'chat_ui.dart';

class ParticipantScreen extends StatelessWidget {
  EnxController enxController = Get.find<EnxController>();

  ParticipantScreen(this.enxController, {Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    print("1211${enxController.userList.length}");

    return Scaffold(
        body: SafeArea(
            child: InkWell(
      onTap: () {
        Get.back();
      },
      child: Obx(() => Column(
            children: [
              Container(
                  width: MediaQuery.of(context).size.width,
                  color: Colors.red,
                  child: Padding(
                    padding: const EdgeInsets.all(12.0),
                    child: Text(
                      'Moderators(${enxController.moderatorsList.length})',
                      style: TextStyle(color: Colors.white),
                    ),
                  )),
              Divider(),
              ListView.separated(
                shrinkWrap: true,
                // Let the ListView know how many items it needs to build.
                itemCount: enxController.moderatorsList.length,
                // Provide a builder function. This is where the magic happens.
                // Convert each item into a widget based on the type of item it is.
                itemBuilder: (context, index) {
                  final item = enxController.moderatorsList[index];

                  return ListTile(
                    leading: Icon(Icons.person),
                    title: Text(
                        enxController.userModelSelf!.clientId == item.clientId
                            ? '${item.name!} (Me)'
                            : item.name!,
                        overflow: TextOverflow.ellipsis,
                        maxLines: 1,
                        style: TextStyle(color: Colors.black, fontSize: 13)),
                    trailing: Wrap(
                      spacing: 8, // space between two icons
                      children: <Widget>[
                        CustomMaterialButton(
                          onPressed: () => {
                            if (enxController.userModelSelf!.clientId ==
                                item.clientId)
                              {enxController.toggleMuteAudio()}
                            else
                              {
                                if (enxController.userModelSelf!.role !=
                                    'participant')
                                  {
                                    if (item.isAudioMuted)
                                      enxController
                                          .toggleHardUnMuteAudio(item.clientId)
                                    else
                                      enxController
                                          .toggleHardMuteAudio(item.clientId)
                                  }
                              }
                          },
                          child: Icon(
                            item.isAudioMuted
                                ? Icons.mic_off_outlined
                                : Icons.mic_none_outlined,
                            color: item.isAudioMuted
                                ? CustomColors.themeColor
                                : Colors.grey,
                          ),
                          elevation: 2.0,
                          padding: EdgeInsets.all(12.0),
                        ), // icon-1
                        CustomMaterialButton(
                          onPressed: () => {
                            if (enxController.userModelSelf!.clientId ==
                                item.clientId)
                              {enxController.toggleMuteVideo()}
                            else
                              {
                                if (enxController.userModelSelf!.role !=
                                    'participant')
                                  {
                                    if (item.isVideoMuted)
                                      enxController
                                          .toggleHardUnMuteVideo(item.clientId)
                                    else
                                      enxController
                                          .toggleHardMuteVideo(item.clientId)
                                  }
                              }
                          },
                          child: Icon(
                              item.isVideoMuted
                                  ? Icons.videocam_off_outlined
                                  : Icons.videocam_outlined,
                              color: item.isVideoMuted
                                  ? CustomColors.themeColor
                                  : Colors.grey),
                          elevation: 2.0,
                          padding: EdgeInsets.all(12.0),
                        ),
                        enxController.userModelSelf!.clientId == item.clientId
                            ? const SizedBox()
                            : CustomMaterialButton(
                                onPressed: () {
                                  enxController.chatType.value = 'private';
                                  Get.to(
                                      () => ChatPage(
                                            enxController,
                                            item.name,
                                            clientId: item.clientId,
                                          ),
                                      transition: Transition.leftToRight,
                                      duration: const Duration(seconds: 1));
                                },
                                child: Image.asset(
                                  "assets/image/chat_icon.png",
                                  package: 'enx_uikit_flutter',
                                  width: 30,
                                  height: 30,
                                ),
                                elevation: 2.0,
                              ),
                        enxController.userModelSelf!.role == 'participant'
                            ? SizedBox()
                            : CustomMaterialButton(
                                onPressed: () => _onCallEnd(
                                    context, [item.clientId],
                                    clientId: enxController
                                                .userModelSelf!.clientId ==
                                            item.clientId
                                        ? enxController.userModelSelf!.clientId
                                        : ''),
                                child: Icon(Icons.call_rounded,
                                    color: Colors.white, size: 20),
                                shape: CircleBorder(),
                                elevation: 2.0,
                                fillColor: CustomColors.themeColor,
                              ), // icon-2
                      ],
                    ),
                  );
                },
                separatorBuilder: (BuildContext context, int index) {
                  return const Divider();
                },
              ),
              Container(
                  width: MediaQuery.of(context).size.width,
                  color: Colors.red,
                  child: Padding(
                    padding: const EdgeInsets.all(12.0),
                    child: Text(
                      'Participants (${enxController.participantsList.length})',
                      style: TextStyle(color: Colors.white),
                    ),
                  )),
              ListView.separated(
                shrinkWrap: true,
                // Let the ListView know how many items it needs to build.
                itemCount: enxController.participantsList.length,
                // Provide a builder function. This is where the magic happens.
                // Convert each item into a widget based on the type of item it is.
                itemBuilder: (context, index) {
                  final item = enxController.participantsList[index];

                  return ListTile(
                    leading: Icon(Icons.person),
                    title: Text(
                        enxController.userModelSelf!.clientId == item.clientId
                            ? '${item.name!} (Me)'
                            : item.name!,
                        overflow: TextOverflow.ellipsis,
                        maxLines: 1,
                        style: TextStyle(color: Colors.black, fontSize: 13)),
                    trailing: Wrap(
                      spacing: 8, // space between two icons
                      children: <Widget>[
                        CustomMaterialButton(
                          onPressed: () => {
                            if (enxController.userModelSelf!.clientId ==
                                item.clientId)
                              {enxController.toggleMuteAudio()}
                            else
                              {
                                if (enxController.userModelSelf!.role !=
                                    'participant')
                                  {
                                    if (item.isAudioMuted)
                                      enxController
                                          .toggleHardUnMuteAudio(item.clientId)
                                    else
                                      enxController
                                          .toggleHardMuteAudio(item.clientId)
                                  }
                              }
                          },
                          child: Icon(
                            item.isAudioMuted
                                ? Icons.mic_off_outlined
                                : Icons.mic_none_outlined,
                            color: item.isAudioMuted
                                ? CustomColors.themeColor
                                : Colors.grey,
                          ),
                          elevation: 2.0,
                          padding: EdgeInsets.all(12.0),
                        ), // icon-1
                        CustomMaterialButton(
                          onPressed: () => {
                            //
                            if (enxController.userModelSelf!.clientId ==
                                item.clientId)
                              {enxController.toggleMuteVideo()}
                            else
                              {
                                if (enxController.userModelSelf!.role !=
                                    'participant')
                                  {
                                    if (item.isVideoMuted)
                                      enxController
                                          .toggleHardUnMuteVideo(item.clientId)
                                    else
                                      enxController
                                          .toggleHardMuteVideo(item.clientId)
                                  }
                              }

                            //
                          },
                          child: Icon(
                              item.isVideoMuted
                                  ? Icons.videocam_off_outlined
                                  : Icons.videocam_outlined,
                              color: item.isVideoMuted
                                  ? CustomColors.themeColor
                                  : Colors.grey),
                          elevation: 2.0,
                          padding: EdgeInsets.all(12.0),
                        ),
                        enxController.userModelSelf!.clientId == item.clientId
                            ? const SizedBox()
                            : CustomMaterialButton(
                                onPressed: () {
                                  enxController.chatType.value = 'private';
                                  Get.to(
                                      () => ChatPage(
                                            enxController,
                                            item.name,
                                            clientId: item.clientId,
                                          ),
                                      transition: Transition.leftToRight,
                                      duration: const Duration(seconds: 1));
                                },
                                child: Image.asset(
                                  "assets/image/chat_icon.png",
                                  package: 'enx_uikit_flutter',
                                 width:   30,
                                  height: 30,
                                ),
                                elevation: 2.0,
                              ),
                        (enxController.userModelSelf!.role == 'participant')
                            ? enxController.userModelSelf!.clientId ==
                                    item.clientId
                                ? CustomMaterialButton(
                                    onPressed: () => _onCallEnd(
                                        context, [item.clientId],
                                        clientId: enxController
                                            .userModelSelf!.clientId),
                                    child: Icon(Icons.call_rounded,
                                        color: Colors.white, size: 20),
                                    shape: CircleBorder(),
                                    elevation: 2.0,
                                    fillColor: CustomColors.themeColor,
                                  )
                                : const SizedBox()
                            : enxController.userModelSelf!.clientId ==
                                    item.clientId
                                ? SizedBox()
                                : CustomMaterialButton(
                                    onPressed: () => _onCallEnd(
                                        context, [item.clientId],
                                        clientId: ''),
                                    child: Icon(Icons.call_rounded,
                                        color: Colors.white, size: 20),
                                    shape: CircleBorder(),
                                    elevation: 2.0,
                                    fillColor: CustomColors.themeColor,
                                  ), // icon-2
                      ],
                    ),
                  );
                },
                separatorBuilder: (BuildContext context, int index) {
                  return const Divider();
                },
              ),
            ],
          )),
    )));
  }

  void _onCallEnd(BuildContext context, List<String> participants,
      {String? clientId}) {
    print('${clientId}');
    if (clientId!.isNotEmpty) {
      enxController.endCall('participantScreen');
    } else {
      enxController.dropCall(participants);
    }
  }
}
